--*
--
-- OBJECTIVE:
--    Check that it is not a bounded error for an explicit call to Adjust or
--    Finalize to propagate an exception.  (Defect Report 8652/0023).
--
--    Note: (RLB, 3/15/01) Existing test C760010 already checks this.
--
-- CHANGE HISTORY:
--    29 JAN 2001   PHL   Initial version
--
--!
with Ada.Exceptions;
use Ada.Exceptions;
with Ada.Finalization;
with C76114A_0;
use C76114A_0;
with Report;
use Report;
procedure C76114A is
begin
    Test
       ("C76114A",
	"CHECK THAT IT IS NOT A BOUNDED ERROR FOR AN EXPLICIT CALL TO ADJUST " &
	   "OR FINALIZE TO PROPAGATE AN EXCEPTION");

    begin
	declare

	    Oce1 : Ctrl := (Ada.Finalization.Controlled with
			    C => Ident_Int (1));
	    Oce2 : Ctrl := (Ada.Finalization.Controlled with
			    C => Ident_Int (-1));
	    Ose1 : Ctrl := (Ada.Finalization.Controlled with
			    C => Ident_Int (2));
	    Ose2 : Ctrl := (Ada.Finalization.Controlled with
			    C => Ident_Int (-2));
	    Ote1 : Ctrl := (Ada.Finalization.Controlled with
			    C => Ident_Int (3));
	    Ote2 : Ctrl := (Ada.Finalization.Controlled with
			    C => Ident_Int (-3));
	    Oue1 : Ctrl := (Ada.Finalization.Controlled with
			    C => Ident_Int (123));
	    Oue2 : Ctrl := (Ada.Finalization.Controlled with
			    C => Ident_Int (234));

	begin

	    begin
		Adjust (Oce1);
	    exception
		when Constraint_Error =>
		    null;
		when E: others =>
		    Failed ("EXCEPTION " & Exception_Name (E) & " RAISED - " &
			    Exception_Information (E) & " - " &
			    Exception_Message (E) &
			    " - ADJUST CONSTRAINT_ERROR");
	    end;


	    begin
		Finalize (Oce2);
	    exception
		when Constraint_Error =>
		    null;
		when E: others =>
		    Failed ("EXCEPTION " & Exception_Name (E) & " RAISED - " &
			    Exception_Information (E) & " - " &
			    Exception_Message (E) &
			    " - FINALIZE CONSTRAINT_ERROR");
	    end;

	    begin
		Adjust (Ose1);
	    exception
		when Storage_Error =>
		    null;
		when E: others =>
		    Failed ("EXCEPTION " & Exception_Name (E) & " RAISED - " &
			    Exception_Information (E) & " - " &
			    Exception_Message (E) & " - ADJUST STORAGE_ERROR");
	    end;

	    begin
		Finalize (Ose2);
	    exception
		when Storage_Error =>
		    null;
		when E: others =>
		    Failed ("EXCEPTION " & Exception_Name (E) & " RAISED - " &
			    Exception_Information (E) & " - " &
			    Exception_Message (E) &
			    " - FINALIZE STORAGE_ERROR");
	    end;

	    begin
		Adjust (Ote1);
	    exception
		when Tasking_Error =>
		    null;
		when E: others =>
		    Failed ("EXCEPTION " & Exception_Name (E) & " RAISED - " &
			    Exception_Information (E) & " - " &
			    Exception_Message (E) & " - ADJUST TASKING_ERROR");
	    end;

	    begin
		Finalize (Ote2);
	    exception
		when Tasking_Error =>
		    null;
		when E: others =>
		    Failed ("EXCEPTION " & Exception_Name (E) & " RAISED - " &
			    Exception_Information (E) & " - " &
			    Exception_Message (E) &
			    " - FINALIZE TASKING_ERROR");
	    end;

	    begin
		Adjust (Oue1);
	    exception
		when User_Error =>
		    null;
		when E: others =>
		    Failed ("EXCEPTION " & Exception_Name (E) & " RAISED - " &
			    Exception_Information (E) & " - " &
			    Exception_Message (E) & " - ADJUST USER_ERROR");
	    end;

	    begin
		Finalize (Oue2);
	    exception
		when User_Error =>
		    null;
		when E: others =>
		    Failed ("EXCEPTION " & Exception_Name (E) & " RAISED - " &
			    Exception_Information (E) & " - " &
			    Exception_Message (E) & " - FINALIZE USER_ERROR");
	    end;
	end;

    exception
	when Program_Error =>
	    -- Normal finalization of the above objects will surely raise
	    -- exceptions, which will be turned into Program_Error.
	    null;
	when E: others =>
	    Failed ("EXCEPTION " & Exception_Name (E) & " RAISED - " &
		    Exception_Information (E) & " - " &
		    Exception_Message (E) & " - NORMAL FINALIZATION");
    end;

    Result;
end C76114A;

with Ada.Finalization;
use Ada.Finalization;
package C76114A_0 is

    User_Error : exception;

    type Ctrl is new Controlled with
	record
	    C : Integer;
	end record;

    procedure Adjust (Obj : in out Ctrl);
    procedure Finalize (Obj : in out Ctrl);

end C76114A_0;

with Report;
use Report;
package body C76114A_0 is

    procedure Adjust (Obj : in out Ctrl) is
    begin
	if Obj.C = Ident_Int (1) then
	    raise Constraint_Error;
	elsif Obj.C = Ident_Int (2) then
	    raise Storage_Error;
	elsif Obj.C = Ident_Int (3) then
	    raise Tasking_Error;
	else
	    raise User_Error;
	end if;
    end Adjust;

    procedure Finalize (Obj : in out Ctrl) is
    begin
	if Obj.C = Ident_Int (-1) then
	    raise Constraint_Error;
	elsif Obj.C = Ident_Int (-2) then
	    raise Storage_Error;
	elsif Obj.C = Ident_Int (-3) then
	    raise Tasking_Error;
	else
	    raise User_Error;
	end if;
    end Finalize;

end C76114A_0;

